require "net/http"
require "openssl"
require "require_all"
require "uri"

require_rel "connector"

# DEPRECATED
# Seiso namespace module
module Seiso
  
  # Seiso connector.
  #
  # Author:: Willie Wheeler
  # Copyright:: Copyright (c) 2014-2015 Expedia, Inc.
  # License:: Apache 2.0
  class Connector
    
    # - settings: a hash containing the following entries:
    #   - host
    #   - port
    #   - use_ssl (default false)
    #   - ignore_cert (default false)
    #   - username
    #   - password
    def initialize(settings)
      host = settings['host']
      port = settings['port']
      use_ssl = settings['use_ssl'] || false
      ignore_cert = settings['ignore_cert'] || false
      username = settings['username']
      password = settings['password']
      
      http = create_http(host, port, use_ssl, ignore_cert)
      
      @connector_v1 = Seiso::Connector::ConnectorV1.new(http, username, password)
    end
    
    # Tests can override this
    def connector_v1
      @connector_v1
    end

    # GET item list
    def get_items(type)
      connector_v1.do_get("/v1/#{type}")
    end
    
    # GET key list. We currently support only type=people.
    # - type: item type
#    def get_keys(type, source, page)
#      raise(NotImplementedError, "get_keys not implemented for type #{type}") if type != "people"
#      path = "/v2/#{type}?view=keys&source=#{source}&page=#{page}"
#      do_get path
#    end
    
    # POST items to Seiso
    def post_items(type, items)
      if type == 'ip-address-roles'
        do_put_ip_address_roles items
      elsif type == 'node-ip-addresses'
        do_put_node_ip_addresses items
      elsif type == 'service-instance-ports'
        do_put_service_instance_ports items
      else
        connector_v1.do_post("/v1/#{type}?mode=batch", items)
      end
    end
    
    # PUT a single item to Seiso.
    # - type: Item type
    # - item: Item
    # - key: Item key
    def put_item(type, item, key)
      connector_v1.do_put("/v1/#{type}/#{key}", item)
    end

    private

    # Create HTTP connector
    # - host: Seiso host
    # - port: Seiso port
    # - use_ssl: Use HTTPS?
    # - ignore_cert: Ignore bad certs?
    def create_http(host, port, use_ssl, ignore_cert)
      http = Net::HTTP.new(host, port)
      if use_ssl
        puts "Using SSL"
        http.use_ssl = true
        if ignore_cert
          puts "Ignoring SSL certificates"
          http.verify_mode = OpenSSL::SSL::VERIFY_NONE
        end
      end
      http
    end
    
    # PUTs a list of IP address roles
    # - roles: IP address roles
    def do_put_ip_address_roles(roles)
      # TODO Implement batch endpoint
      roles.each do |r|
        si_key = r['serviceInstance']['key']
        name = r['name']
        connector_v1.do_put("/v1/service-instances/#{si_key}/ip-address-roles/#{name}", r)
      end
    end

    # PUTs a list of node IP addresses
    # - nips: Node IP addresses
    def do_put_node_ip_addresses(nips)
      # TODO Implement batch endpoint
      nips.each do |nip|
        node_name = nip['node']['name']
        ip_address = nip['ipAddress']
        connector_v1.do_put("/v1/nodes/#{node_name}/ip-addresses/#{ip_address}", nip)
      end
    end

    # PUTs a list of service instance ports
    # - ports: Service instance ports
    def do_put_service_instance_ports(ports)
      # TODO Implement batch endpoint
      ports.each do |p|
        si_key = p['serviceInstance']['key']
        number = p['number']
        connector_v1.do_put("/v1/service-instances/#{si_key}/ports/#{number}", p)
      end
    end
  end
end
