require "uri"

module Seiso
  class Connector

    # Seiso API v1 URI factory.
    # 
    # URI templating (embodied here) is a temporary strategy for obtaining URIs. The plan is
    # to adopt something like a Traverson-style strategy.
    # 
    # Author:: Willie Wheeler (mailto:wwheeler@expedia.com)
    # Copyright:: Copyright (c) 2014-2015 Expedia, Inc.
    # License:: Apache 2.0
    class UriFactoryV1

      def initialize(base_uri)
        @base_uri = base_uri
      end

      def nodes_uri(batch)
        if batch
          uri "/nodes?mode=batch"
        else
          uri "/nodes"
        end
      end

      def node_uri(name)
        uri "/nodes/#{name}"
      end
      
      def node_ip_address_uri(node_name, ip_address)
        uri "/nodes/#{node_name}/ip-addresses/#{ip_address}"
      end

      def services_uri(batch)
        if batch
          uri "/services?mode=batch"
        else
          uri "/services"
        end
      end

      def service_instances_uri(batch)
        if batch
          uri "/service-instances?mode=batch"
        else
          uri "/service-instances"
        end
      end

      def machines_uri(batch)
        if batch
          uri "/machines?mode=batch"
        else
          uri "/machines"
        end
      end

      def machine_uri(name)
        uri "/machines/#{name}"
      end

      private

      def uri(path)
        URI.parse "#{@base_uri}#{path}"
      end
      
    end # class UriFactoryV1
  end # class Connector
end # module Seiso
