require "uri"

module Seiso
  class Connector

    # Seiso API v2 URI factory.
    # 
    # URI templating (embodied here) is a temporary strategy for obtaining URIs. The plan is
    # to adopt something like a Traverson-style strategy.
    # 
    # Author:: Willie Wheeler (mailto:wwheeler@expedia.com)
    # Copyright:: Copyright (c) 2014-2015 Expedia, Inc.
    # License:: Apache 2.0
    class UriFactoryV2

      def initialize(base_uri)
        @base_uri = base_uri
      end

      def doc_links_uri(service_key)
        uri "/services/#{service_key}/doc-links"
      end

      def service_instance_uri(key)
        uri "/service-instances/#{key}"
      end

      def service_instance_seyren_checks_uri(key)
        uri "/service-instances/#{key}/seyren-checks"
      end

      def service_instance_seyren_check_uri(service_instance_key, check_id)
        uri "/service-instances/#{key}/seyren-checks/#{check_id}"
      end

      def seyren_checks_uri()
        uri "/seyren-checks"
      end

      def find_seyren_check_uri(seyren_base_url, seyren_id)
        uri "/seyren-checks/search/find-by-base-url-and-id?url=#{seyren_base_url}&id=#{seyren_id}"
      end

      def uri(path)
        URI.parse "#{@base_uri}#{path}"
      end
      
    end # class UriFactoryV2
  end # class Connector
end # module Seiso
